"""
This library implements various methods for working with the Google KMS APIs.

## Installation

```console
$ pip install --upgrade gcloud-rest-kms
```

## Usage

We're still working on more complete documentation, but roughly you can do:

```python
from gcloud.rest.kms import KMS
from gcloud.rest.kms import decode
from gcloud.rest.kms import encode

kms = KMS('my-kms-project', 'my-keyring', 'my-key-name')

# encrypt
plaintext = b'the-best-animal-is-the-aardvark'
ciphertext = kms.encrypt(encode(plaintext))

# decrypt
assert decode(kms.decrypt(ciphertext)) == plaintext

# close the HTTP session
# Note that other options include:
# * providing your own session: `KMS(.., session=session)`
# * using a context manager: `with KMS(..) as kms:`
kms.close()
```

## Emulators

For testing purposes, you may want to use `gcloud-rest-kms` along with a local
emulator. Setting the `$KMS_EMULATOR_HOST` environment variable to the address
of your emulator should be enough to do the trick.
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals
import warnings
with warnings.catch_warnings():
    warnings.simplefilter("ignore")
    from future import standard_library
standard_library.install_aliases()
from pkg_resources import get_distribution
__version__ = get_distribution('gcloud-rest-kms').version

from gcloud.rest.kms.kms import KMS
from gcloud.rest.kms.kms import SCOPES
from gcloud.rest.kms.utils import decode
from gcloud.rest.kms.utils import encode


__all__ = ['__version__', 'decode', 'encode', 'KMS', 'SCOPES']



from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals
import warnings
with warnings.catch_warnings():
    warnings.simplefilter("ignore")
    from future import standard_library
standard_library.install_aliases()
import base64
from typing import Union


def decode(payload     )         :
    """
    https://en.wikipedia.org/wiki/Base64#URL_applications modified Base64
    for URL variants exist, where the + and / characters of standard
    Base64 are respectively replaced by - and _

    Does not make any assumptions about encoding -- if you're encoding a bytes
    payload then `foo == decode(encode(foo))`, but if `foo` is a string you'll
    need to `.decode()` manually according to your expected encoding scheme.
    """
    variant = payload.replace('-', '+').replace('_', '/')
    return base64.b64decode(variant)


def encode(payload                   )       :
    """
    https://en.wikipedia.org/wiki/Base64#URL_applications modified Base64
    for URL variants exist, where the + and / characters of standard
    Base64 are respectively replaced by - and _
    """
    if not isinstance(payload, bytes):
        payload = payload.encode('utf-8')

    encoded = base64.b64encode(payload)
    return encoded.replace(b'+', b'-').replace(b'/', b'_').decode('utf-8')



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['gcloud', 'gcloud.rest', 'gcloud.rest.kms']

package_data = \
{'': ['*']}

install_requires = \
['gcloud-rest-auth>=3.1.0,<5.0.0']

setup_kwargs = {
    'name': 'gcloud-rest-kms',
    'version': '4.1.0',
    'description': 'Python Client for Google Cloud KMS',
    'long_description': '(Asyncio OR Threadsafe) Python Client for Google Cloud KMS\n==========================================================\n\n    This is a shared codebase for ``gcloud-rest-kms`` and ``gcloud-rest-kms``\n\n|pypi| |pythons-aio| |pythons-rest|\n\nInstallation\n------------\n\n.. code-block:: console\n\n    $ pip install --upgrade gcloud-{aio,rest}-kms\n\nUsage\n-----\n\nSee `our docs`_.\n\nContributing\n------------\n\nPlease see our `contributing guide`_.\n\n.. _contributing guide: https://github.com/talkiq/gcloud-rest/blob/master/.github/CONTRIBUTING.rst\n.. _our docs: https://talkiq.github.io/gcloud-rest\n\n.. |pypi| image:: https://img.shields.io/pypi/v/gcloud-rest-kms.svg?style=flat-square\n    :alt: Latest PyPI Version (gcloud-rest-kms)\n    :target: https://pypi.org/project/gcloud-rest-kms/\n\n.. |pythons-aio| image:: https://img.shields.io/pypi/pyversions/gcloud-rest-kms.svg?style=flat-square&label=python (aio)\n    :alt: Python Version Support (gcloud-rest-kms)\n    :target: https://pypi.org/project/gcloud-rest-kms/\n\n.. |pythons-rest| image:: https://img.shields.io/pypi/pyversions/gcloud-rest-kms.svg?style=flat-square&label=python (rest)\n    :alt: Python Version Support (gcloud-rest-kms)\n    :target: https://pypi.org/project/gcloud-rest-kms/\n',
    'author': 'Vi Engineering',
    'author_email': 'voiceai-eng@dialpad.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/talkiq/gcloud-aio',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*, !=3.5.*, !=3.6.*',
}


setup(**setup_kwargs)



"""
An asynchronous client for Google Cloud KMS
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals
import warnings
with warnings.catch_warnings():
    warnings.simplefilter("ignore")
    from future import standard_library
standard_library.install_aliases()
from builtins import object
import json
import os
from typing import Any
from typing import AnyStr
from typing import Dict
from typing import IO
from typing import Optional
from typing import Tuple
from typing import Union

from gcloud.rest.auth import SyncSession  # pylint: disable=no-name-in-module
from gcloud.rest.auth import BUILD_GCLOUD_REST  # pylint: disable=no-name-in-module
from gcloud.rest.auth import Token  # pylint: disable=no-name-in-module

# Selectively load libraries based on the package
if BUILD_GCLOUD_REST:
    from requests import Session
else:
    from aiohttp import ClientSession as Session  # type: ignore[assignment]


SCOPES = [
    'https://www.googleapis.com/auth/cloudkms',
]


def init_api_root(api_root               )                    :
    if api_root:
        return True, api_root

    host = os.environ.get('KMS_EMULATOR_HOST')
    if host:
        return True, 'http://{}/v1'.format((host))

    return False, 'https://cloudkms.googleapis.com/v1'


class KMS(object):
    #_api_root: str
    #_api_is_dev: bool

    def __init__(
            self, keyproject     , keyring     , keyname     ,
            service_file                                   = None,
            location      = 'global', session                    = None,
            token                  = None, api_root                = None,
    )        :
        self._api_is_dev, self._api_root = init_api_root(api_root)
        self._api_root = (
            '{}/projects/{}/locations/{}/'
            'keyRings/{}/cryptoKeys/{}'.format((self._api_root), (keyproject), (location), (keyring), (keyname))
        )

        self.session = SyncSession(session)
        self.token = token or Token(
            service_file=service_file,
            session=self.session.session,  # type: ignore[arg-type]
            scopes=SCOPES,
        )

    def headers(self)                  :
        if self._api_is_dev:
            return {'Content-Type': 'application/json'}

        token = self.token.get()
        return {
            'Authorization': 'Bearer {}'.format((token)),
            'Content-Type': 'application/json',
        }

    # https://cloud.google.com/kms/docs/reference/rest/v1/projects.locations.keyRings.cryptoKeys/decrypt
    def decrypt(
        self, ciphertext     ,
        session                    = None,
    )       :
        url = '{}:decrypt'.format((self._api_root))
        body = json.dumps({
            'ciphertext': ciphertext,
        }).encode('utf-8')

        s = SyncSession(session) if session else self.session
        resp = s.post(url, headers=self.headers(), data=body)

        plaintext      = (resp.json())['plaintext']
        return plaintext

    # https://cloud.google.com/kms/docs/reference/rest/v1/projects.locations.keyRings.cryptoKeys/encrypt
    def encrypt(
        self, plaintext     ,
        session                    = None,
    )       :
        url = '{}:encrypt'.format((self._api_root))
        body = json.dumps({
            'plaintext': plaintext,
        }).encode('utf-8')

        s = SyncSession(session) if session else self.session
        resp = s.post(url, headers=self.headers(), data=body)

        ciphertext      = (resp.json())['ciphertext']
        return ciphertext

    def close(self)        :
        self.session.close()

    def __enter__(self)         :
        return self

    def __exit__(self, *args     )        :
        self.close()
